@@ -58,7 +58,7 @@
 
 
 /* argument parsing */
-#define MKHOMEDIR_DEBUG      020	/* keep quiet about things */
+#define MKHOMEDIR_DEBUG      020	/* be verbose about things */
 #define MKHOMEDIR_QUIET      040	/* keep quiet about things */
 
 static unsigned int UMask = 0022;
@@ -78,6 +78,8 @@ _pam_parse (const pam_handle_t *pamh, int flags, int argc, const char **argv)
    {
       if (!strcmp(*argv, "silent")) {
 	 ctrl |= MKHOMEDIR_QUIET;
+      } else if (!strcmp(*argv, "debug")) {
+         ctrl |= MKHOMEDIR_DEBUG;
       } else if (!strncmp(*argv,"umask=",6)) {
 	 UMask = strtol(*argv+6,0,0);
       } else if (!strncmp(*argv,"skel=",5)) {
@@ -103,7 +105,7 @@ rec_mkdir (const char *dir, mode_t mode)
 
   cp = strrchr (parent, '/');
 
-  if (cp != NULL)
+  if (cp != NULL && cp != parent)
     {
       struct stat st;
 
@@ -137,12 +139,13 @@ create_homedir (pam_handle_t * pamh, int ctrl,
 
    /* Mention what is happening, if the notification fails that is OK */
    if ((ctrl & MKHOMEDIR_QUIET) != MKHOMEDIR_QUIET)
-      (void) pam_info(pamh, "Creating directory '%s'.", dest);
+      pam_info(pamh, _("Creating directory '%s'."), dest);
 
    /* Create the new directory */
    if (rec_mkdir (dest,0755) != 0)
    {
-      pam_syslog(pamh, LOG_DEBUG, "unable to create directory %s: %m", dest);
+      pam_error(pamh, _("Unable to create directory %s: %m"), dest);
+      pam_syslog(pamh, LOG_ERR, "unable to create directory %s: %m", dest);
       return PAM_PERM_DENIED;
    }
 
@@ -341,7 +344,20 @@ create_homedir (pam_handle_t * pamh, int ctrl,
 
 	 return PAM_PERM_DENIED;
       }
-      stat(newsource,&St);
+      if (stat(newsource,&St) != 0)
+	{
+	  pam_syslog(pamh, LOG_DEBUG, "unable to stat src file %s: %m",
+		     newsource);
+	  close(SrcFd);
+	  closedir(D);
+
+#ifndef PATH_MAX
+	  free(newsource); newsource = NULL;
+	  free(newdest); newdest = NULL;
+#endif
+
+	  return PAM_PERM_DENIED;
+	}
 
       /* Open the dest file */
       if ((DestFd = open(newdest,O_WRONLY | O_TRUNC | O_CREAT,0600)) < 0)
