@@ -94,7 +94,7 @@ int _unix_run_verify_binary(pam_handle_t *pamh, unsigned int ctrl,
     size_t i=0;
     struct rlimit rlim;
     static char *envp[] = { NULL };
-    char *args[] = { NULL, NULL, NULL, NULL, NULL };
+    char *args[] = { NULL, NULL, NULL, NULL };
 
     close(0); close(1);
     /* reopen stdin as pipe */
@@ -121,7 +121,6 @@ int _unix_run_verify_binary(pam_handle_t *pamh, unsigned int ctrl,
     args[0] = x_strdup(CHKPWD_HELPER);
     args[1] = x_strdup(user);
     args[2] = x_strdup("chkexpiry");
-    args[3] = x_strdup(on(UNIX_CLUSTER, ctrl) ? "cluster" : "local");
 
     execve(CHKPWD_HELPER, args, envp);
 
@@ -140,6 +139,9 @@ int _unix_run_verify_binary(pam_handle_t *pamh, unsigned int ctrl,
       if (rc<0) {
 	pam_syslog(pamh, LOG_ERR, "unix_chkpwd waitpid returned %d: %m", rc);
 	retval = PAM_AUTH_ERR;
+      } else if (!WIFEXITED(retval)) {
+        pam_syslog(pamh, LOG_ERR, "unix_chkpwd abnormal exit: %d", retval);
+        retval = PAM_AUTH_ERR;
       } else {
 	retval = WEXITSTATUS(retval);
         rc = pam_modutil_read(fds[0], buf, sizeof(buf) - 1);
@@ -197,7 +199,7 @@ PAM_EXTERN int pam_sm_acct_mgmt(pam_handle_t * pamh, int flags,
 		return PAM_USER_UNKNOWN;
 	}
 
-	retval = get_account_info(pamh, ctrl, uname, &pwent, &spent);
+	retval = get_account_info(pamh, uname, &pwent, &spent);
 	if (retval == PAM_USER_UNKNOWN) {
 		pam_syslog(pamh, LOG_ALERT,
 			 "could not identify user (from getpwnam(%s))",
@@ -251,6 +253,9 @@ PAM_EXTERN int pam_sm_acct_mgmt(pam_handle_t * pamh, int flags,
 		_make_remark(pamh, ctrl, PAM_ERROR_MSG,
 			_("Your account has expired; please contact your system administrator"));
 		break;
+	case PAM_AUTHTOK_ERR:
+		retval = PAM_SUCCESS;
+		/* fallthrough */
 	case PAM_SUCCESS:
 		if (daysleft >= 0) {
 			pam_syslog(pamh, LOG_DEBUG,
