@@ -82,7 +82,6 @@ static int selinux_enabled=-1;
 #include "support.h"
 #include "passverify.h"
 #include "bigcrypt.h"
-#include "cluster.h"
 
 #if !((__GLIBC__ == 2) && (__GLIBC_MINOR__ >= 1))
 extern int getrpcport(const char *host, unsigned long prognum,
@@ -167,7 +166,7 @@ static int _unix_run_update_binary(pam_handle_t *pamh, unsigned int ctrl, const
         size_t i=0;
         struct rlimit rlim;
 	static char *envp[] = { NULL };
-	char *args[] = { NULL, NULL, NULL, NULL, NULL, NULL, NULL };
+	char *args[] = { NULL, NULL, NULL, NULL, NULL, NULL };
         char buffer[16];
 
 	/* XXX - should really tidy up PAM here too */
@@ -194,8 +193,10 @@ static int _unix_run_update_binary(pam_handle_t *pamh, unsigned int ctrl, const
 	args[0] = x_strdup(UPDATE_HELPER);
 	args[1] = x_strdup(user);
 	args[2] = x_strdup("update");
-	args[3] = x_strdup(on(UNIX_SHADOW, ctrl) ? "1" : "0");
-	args[5] = x_strdup(on(UNIX_CLUSTER, ctrl) ? "cluster" : "local");
+	if (on(UNIX_SHADOW, ctrl))
+		args[3] = x_strdup("1");
+	else
+		args[3] = x_strdup("0");
 
         snprintf(buffer, sizeof(buffer), "%d", remember);
         args[4] = x_strdup(buffer);
@@ -224,8 +225,11 @@ static int _unix_run_update_binary(pam_handle_t *pamh, unsigned int ctrl, const
 	rc=waitpid(child, &retval, 0);  /* wait for helper to complete */
 	if (rc<0) {
 	  pam_syslog(pamh, LOG_ERR, "unix_update waitpid failed: %m");
-	  retval = PAM_AUTH_ERR;
-	} else {
+	  retval = PAM_AUTHTOK_ERR;
+	} else if (!WIFEXITED(retval)) {
+          pam_syslog(pamh, LOG_ERR, "unix_update abnormal exit: %d", retval);
+          retval = PAM_AUTHTOK_ERR;
+        } else {
 	  retval = WEXITSTATUS(retval);
 	}
     } else {
@@ -243,8 +247,7 @@ static int _unix_run_update_binary(pam_handle_t *pamh, unsigned int ctrl, const
 }
 #endif
 
-static int check_old_password(unsigned int ctrl,
-			      const char *forwho, const char *newpass)
+static int check_old_password(const char *forwho, const char *newpass)
 {
 	static char buf[16384];
 	char *s_luser, *s_uid, *s_npas, *s_pas;
@@ -299,7 +302,7 @@ static int _do_setpass(pam_handle_t* pamh, const char *forwho,
 		goto done;
 	}
 
-	if (on(UNIX_NIS, ctrl) && _unix_comesfromsource(pamh, ctrl, forwho, 0, 1)) {
+	if (on(UNIX_NIS, ctrl) && _unix_comesfromsource(pamh, forwho, 0, 1)) {
 	    if ((master=getNISserver(pamh)) != NULL) {
 		struct timeval timeout;
 		struct yppasswd yppwd;
@@ -308,7 +311,7 @@ static int _do_setpass(pam_handle_t* pamh, const char *forwho,
 		enum clnt_stat err;
 
 		/* Unlock passwd file to avoid deadlock */
-		unlock_pwdf(ctrl);
+		unlock_pwdf();
 		unlocked = 1;
 
 		/* Initialize password information */
@@ -367,34 +370,34 @@ static int _do_setpass(pam_handle_t* pamh, const char *forwho,
 	    }
 	}
 
-	if (_unix_comesfromsource(pamh, ctrl, forwho, 1, 0)) {
+	if (_unix_comesfromsource(pamh, forwho, 1, 0)) {
 		if(unlocked) {
-			if (lock_pwdf(ctrl) != PAM_SUCCESS) {
+			if (lock_pwdf() != PAM_SUCCESS) {
 				return PAM_AUTHTOK_LOCK_BUSY;
 			}
 		}
 #ifdef WITH_SELINUX
-	        if (unix_selinux_confined(ctrl))
+	        if (unix_selinux_confined())
 			  return _unix_run_update_binary(pamh, ctrl, forwho, fromwhat, towhat, remember);
 #endif
 		/* first, save old password */
-		if (save_old_password(ctrl, forwho, fromwhat, remember)) {
+		if (save_old_password(forwho, fromwhat, remember)) {
 			retval = PAM_AUTHTOK_ERR;
 			goto done;
 		}
 		if (on(UNIX_SHADOW, ctrl) || is_pwd_shadowed(pwd)) {
-			retval = unix_update_shadow(pamh, ctrl, forwho, towhat);
+			retval = unix_update_shadow(pamh, forwho, towhat);
 			if (retval == PAM_SUCCESS)
 				if (!is_pwd_shadowed(pwd))
-					retval = unix_update_passwd(pamh, ctrl, forwho, "x");
+					retval = unix_update_passwd(pamh, forwho, "x");
 		} else {
-			retval = unix_update_passwd(pamh, ctrl, forwho, towhat);
+			retval = unix_update_passwd(pamh, forwho, towhat);
 		}
 	}
 
 
 done:
-	unlock_pwdf(ctrl);
+	unlock_pwdf();
 
 	return retval;
 }
@@ -406,7 +409,7 @@ static int _unix_verify_shadow(pam_handle_t *pamh, const char *user, unsigned in
 	int daysleft;
 	int retval;
 
-	retval = get_account_info(pamh, ctrl, user, &pwent, &spent);
+	retval = get_account_info(pamh, user, &pwent, &spent);
 	if (retval == PAM_USER_UNKNOWN) {
 		return retval;
 	}
@@ -466,7 +469,7 @@ static int _pam_unix_approve_pass(pam_handle_t * pamh
 		  remark = _("You must choose a longer password");
 		D(("length check [%s]", remark));
 		if (on(UNIX_REMEMBER_PASSWD, ctrl)) {
-			if ((retval = check_old_password(ctrl, user, pass_new)) == PAM_AUTHTOK_ERR)
+			if ((retval = check_old_password(user, pass_new)) == PAM_AUTHTOK_ERR)
 			  remark = _("Password has been already used. Choose another.");
 			if (retval == PAM_ABORT) {
 				pam_syslog(pamh, LOG_ERR, "can't open %s file to check old passwords",
@@ -533,14 +536,14 @@ PAM_EXTERN int pam_sm_chauthtok(pam_handle_t * pamh, int flags,
 	 * getpwnam() doesn't tell you *where* the information it gives you
 	 * came from, nor should it.  That's our job.
 	 */
-	if (_unix_comesfromsource(pamh, ctrl, user, 1, on(UNIX_NIS, ctrl)) == 0) {
+	if (_unix_comesfromsource(pamh, user, 1, on(UNIX_NIS, ctrl)) == 0) {
 		pam_syslog(pamh, LOG_DEBUG,
-			 "user \"%s\" does not exist in %s%s",
-			 user, PASSWD_FILE, on(UNIX_NIS, ctrl) ? " or NIS" : "");
+			 "user \"%s\" does not exist in /etc/passwd%s",
+			 user, on(UNIX_NIS, ctrl) ? " or NIS" : "");
 		return PAM_USER_UNKNOWN;
 	} else {
 		struct passwd *pwd;
-		_unix_getpwnam(pamh, ctrl, user, 1, 1, &pwd);
+		_unix_getpwnam(pamh, user, 1, 1, &pwd);
 		if (pwd == NULL) {
 			pam_syslog(pamh, LOG_DEBUG,
 				"user \"%s\" has corrupted passwd entry",
@@ -707,7 +710,7 @@ PAM_EXTERN int pam_sm_chauthtok(pam_handle_t * pamh, int flags,
 			pass_new = pass_old = NULL;	/* tidy up */
 			return retval;
 		}
-		if (lock_pwdf(ctrl) != PAM_SUCCESS) {
+		if (lock_pwdf() != PAM_SUCCESS) {
 			return PAM_AUTHTOK_LOCK_BUSY;
 		}
 
@@ -715,7 +718,7 @@ PAM_EXTERN int pam_sm_chauthtok(pam_handle_t * pamh, int flags,
 			retval = _unix_verify_password(pamh, user, pass_old, ctrl);
 			if (retval != PAM_SUCCESS) {
 				pam_syslog(pamh, LOG_NOTICE, "user password changed by another process");
-				unlock_pwdf(ctrl);
+				unlock_pwdf();
 				return retval;
 			}
 		}
@@ -723,7 +726,7 @@ PAM_EXTERN int pam_sm_chauthtok(pam_handle_t * pamh, int flags,
 		retval = _unix_verify_shadow(pamh, user, ctrl);
 		if (retval != PAM_SUCCESS) {
 			pam_syslog(pamh, LOG_NOTICE, "user shadow entry expired");
-			unlock_pwdf(ctrl);
+			unlock_pwdf();
 			return retval;
 		}
 
@@ -732,7 +735,7 @@ PAM_EXTERN int pam_sm_chauthtok(pam_handle_t * pamh, int flags,
 			pam_syslog(pamh, LOG_NOTICE,
 			         "new password not acceptable 2");
 			pass_new = pass_old = NULL;	/* tidy up */
-			unlock_pwdf(ctrl);
+			unlock_pwdf();
 			return retval;
 		}
 
@@ -745,12 +748,12 @@ PAM_EXTERN int pam_sm_chauthtok(pam_handle_t * pamh, int flags,
 		 * First we encrypt the new password.
 		 */
 
-		tpass = create_password_hash(pamh, pass_new, ctrl, rounds);
+		tpass = create_password_hash(pass_new, ctrl, rounds);
 		if (tpass == NULL) {
 			pam_syslog(pamh, LOG_CRIT,
 				"out of memory for password");
 			pass_new = pass_old = NULL;	/* tidy up */
-			unlock_pwdf(ctrl);
+			unlock_pwdf();
 			return PAM_BUF_ERR;
 		}
 
